﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SS = System.Speech;
using Microsoft.Kinect;
using Microsoft.Speech;
using System.IO;


namespace TestKinectDictation
{
    class Program
    {
        static void Main(string[] args)
        {
            // Obtain a KinectSensor if any are available
            KinectSensor kinect = (from sensorToCheck in KinectSensor.KinectSensors 
                                   where sensorToCheck.Status == KinectStatus.Connected 
                                   select sensorToCheck).FirstOrDefault();
            if (kinect == null)
            {
                throw new Exception("There is not a Kinect connected");
            }
            Console.WriteLine("I've found a Kinect sensor to use.");
            kinect.Start();
            KinectAudioSource source = kinect.AudioSource;

            source.EchoCancellationMode = EchoCancellationMode.None; 
            source.AutomaticGainControlEnabled = false; 
            SS.Recognition.RecognizerInfo ri = SS.Recognition.SpeechRecognitionEngine.InstalledRecognizers().FirstOrDefault();
            using (var spRecEng = new SS.Recognition.SpeechRecognitionEngine(ri.Id))
            {
                //let's create a "normal" grammar
                var phrases = new SS.Recognition.Choices();
                phrases.Add("this is a test");
                phrases.Add("something to say");
                phrases.Add("merry christmas");

                SS.Recognition.GrammarBuilder gb = new SS.Recognition.GrammarBuilder(phrases);
                SS.Recognition.Grammar normGrammar = new SS.Recognition.Grammar(gb);

                // now create the dictation grammar
                SS.Recognition.DictationGrammar customDictationGrammar = new SS.Recognition.DictationGrammar();
                customDictationGrammar.Name = "Dictation";
                customDictationGrammar.Enabled = true;

                // now we load the grammars into our recognition engine
                spRecEng.LoadGrammar(normGrammar);
                spRecEng.LoadGrammar(customDictationGrammar);

                // create an anonymous delegate to handle the SpeechRecognized event
                spRecEng.SpeechRecognized += (s, sargs) => Console.Write(sargs.Result.Text + Environment.NewLine);

                using (Stream s = source.Start())
                {
                    // point the recogintion engine to the Kinect hardware to provide the audio input
                    spRecEng.SetInputToAudioStream(s, 
                                                     new System.Speech.AudioFormat.SpeechAudioFormatInfo(System.Speech.AudioFormat.EncodingFormat.Pcm, 
                                                                                                         16000, 
                                                                                                         16, 
                                                                                                         1, 
                                                                                                         32000, 
                                                                                                         2, 
                                                                                                         null));

                    Console.WriteLine("I am listening. When you are ready for me to stop either press the <enter> key.");

                    spRecEng.RecognizeAsync(System.Speech.Recognition.RecognizeMode.Multiple);

                    Console.ReadLine();
                    Console.WriteLine("Ok. I am shutting down.");
                    spRecEng.RecognizeAsyncCancel();
                }
            }
            kinect.Stop();
        }
    }
}
